package com.supervision.manage.controller.config;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.supervision.manage.pojo.vo.PhysicalToolVO;
import com.supervision.manage.service.PhysicalToolManageService;
import com.supervision.model.ConfigPhysicalTool;
import com.supervision.vo.ask.ConfigPhysicalToolResVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Api(tags = "体格检查工具管理")
@RestController
@RequestMapping("physicalTool")
@RequiredArgsConstructor
public class PhysicalToolManageController {

    private final PhysicalToolManageService physicalToolManageService;

    @ApiOperation(value = "查询体格检查的工具列表")
    @GetMapping("queryPhysicalToolList")
    public List<ConfigPhysicalToolResVO> queryPhysicalToolList() {

        return physicalToolManageService.queryPhysicalToolList();
    }

    @ApiOperation("体格检查分页查询")
    @GetMapping("queryPhysicalPage")
    public IPage<ConfigPhysicalTool> queryConfigPhysicalToolPage(String toolName, String type, Integer requireLocation,
                                                                 Integer pageNum, Integer pageSize) {
        return physicalToolManageService.queryConfigPhysicalToolPage(toolName, type, requireLocation, pageNum, pageSize);
    }

    @ApiModelProperty("查询体格检查工具详细信息")
    @GetMapping("queryConfigPhysicalToolDetail")
    public PhysicalToolVO queryConfigPhysicalToolDetail(String id) {
        return physicalToolManageService.queryConfigPhysicalToolDetail(id);
    }



    @ApiOperation("保存体格工具")
    @PostMapping("saveConfigPhysicalTool")
    public void saveConfigPhysicalTool(@RequestBody PhysicalToolVO physicalToolVO) {
        physicalToolManageService.saveConfigPhysicalTool(physicalToolVO);
    }

    @ApiOperation("修改体格工具")
    @PostMapping("modifyConfigPhysicalTool")
    public void modifyConfigPhysicalTool(@RequestBody PhysicalToolVO physicalToolVO) {
        physicalToolManageService.modifyConfigPhysicalTool(physicalToolVO);
    }

    @ApiOperation("删除体格工具")
    @GetMapping("deleteConfigPhysicalTool")
    public void deleteConfigPhysicalTool(String id) {
        physicalToolManageService.deleteConfigPhysicalTool(id);
    }

}
