package com.supervision.manage.controller.config;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.supervision.manage.service.ConfigTreatmentPlanManageService;
import com.supervision.model.ConfigDrug;
import com.supervision.vo.manage.TreatmentPlanTreeNode;
import com.supervision.model.ConfigTreatmentPlan;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@Api(tags = "处置计划管理")
@RestController
@RequestMapping("treatmentPlan")
@RequiredArgsConstructor
public class TreatmentPlanManageController {

    private final ConfigTreatmentPlanManageService configTreatmentPlanManageService;

    @ApiOperation("查询处置计划树")
    @GetMapping("/queryTree")
    public List<TreatmentPlanTreeNode> queryTree(@ApiParam("处置方式")@RequestParam(required = false) Integer disposalMethod) {

        return configTreatmentPlanManageService.queryTree(disposalMethod);

    }

  /*  @ApiOperation("查询一级处置计划")
    @GetMapping("/queryFirstMeasures")
    public List<ConfigTreatmentPlan> queryFirstMeasures(@ApiParam("处置方式")@RequestParam(required = false) Integer disposalMethod,
                                                          @ApiParam("处置计划id")@RequestParam(required = false) String disposalPlanId) {

        return configTreatmentPlanManageService.queryFirstMeasures(disposalMethod);

    }*/


    @ApiOperation("根据id查询处置计划详情")
    @GetMapping("/findById")
    public ConfigTreatmentPlan findById(@ApiParam("主键") @RequestParam String id) {

        return configTreatmentPlanManageService.findById(id);

    }

    @ApiOperation("分页查询处置计划列表")
    @GetMapping("/queryPageList")
    public IPage<? extends ConfigTreatmentPlan> queryPageList(@RequestParam(defaultValue = "1")Integer pageNum,
                                                  @RequestParam(defaultValue = "10")Integer pageSize,
                                                  @ApiParam("处置方式(0门诊 1住院)")
                                                 @RequestParam(required = false) Integer disposalMethod) {

        return configTreatmentPlanManageService.queryPageList(pageNum, pageSize, disposalMethod);

    }

    @ApiOperation("保存处置计划")
    @PostMapping("/save")
    public String saveTreatmentPlan(@RequestBody ConfigTreatmentPlan configTreatmentPlan) {

        ConfigTreatmentPlan result = configTreatmentPlanManageService.saveTreatmentPlan(configTreatmentPlan);
        return result.getId();
    }

    @ApiOperation("删除处置计划")
    @DeleteMapping("/delete")
    public boolean deleteTreatmentPlan(@RequestParam("id") String id) {

        return configTreatmentPlanManageService.deleteTreatmentPlan(id);
    }

    @ApiOperation("修改处置计划")
    @PutMapping("/update")
    public boolean updateTreatmentPlan(@RequestBody ConfigTreatmentPlan configDrug) {

        return configTreatmentPlanManageService.updateTreatmentPlan(configDrug);
    }


}
