package com.supervision.manage.service.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.supervision.manage.constant.DisposalPlanEnum;
import com.supervision.manage.service.ConfigTreatmentPlanManageService;
import com.supervision.model.ConfigTreatmentPlan;
import com.supervision.model.DiseaseTreatmentPlan;
import com.supervision.service.ConfigTreatmentPlanService;
import com.supervision.service.DiseaseTreatmentPlanService;
import com.supervision.vo.manage.TreatmentPlanTreeNode;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

@Service
@RequiredArgsConstructor
public class ConfigTreatmentPlanManageServiceImpl implements ConfigTreatmentPlanManageService {

    private final ConfigTreatmentPlanService configTreatmentPlanService;

    private final DiseaseTreatmentPlanService diseaseTreatmentPlanService;

    @Override
    public List<TreatmentPlanTreeNode> queryTree(Integer disposalMethod) {

        return configTreatmentPlanService.queryTree(disposalMethod);
    }


    @Override
    public ConfigTreatmentPlan findById(String id) {

        Assert.notEmpty(id,"id 不允许为空");
        return configTreatmentPlanService.getById(id);

    }

    @Override
    public IPage<? extends ConfigTreatmentPlan> queryPageList(Integer pageNum, Integer pageSize, Integer disposalMethod) {
        LambdaQueryChainWrapper<ConfigTreatmentPlan> wrapper = configTreatmentPlanService.lambdaQuery()
                .eq(Objects.nonNull(disposalMethod), ConfigTreatmentPlan::getDisposalMethod, disposalMethod)
                .orderByAsc(ConfigTreatmentPlan::getCreateTime);

        return configTreatmentPlanService.page(new Page<>(pageNum, pageSize), wrapper.getWrapper());
    }

    @Override
    public ConfigTreatmentPlan saveTreatmentPlan(ConfigTreatmentPlan treatmentPlan) {

        saveAssert(treatmentPlan);

        configTreatmentPlanService.save(treatmentPlan);

        return treatmentPlan;
    }

    @Override
    public boolean deleteTreatmentPlan(String id) {

        Assert.notEmpty(id,"id 不允许为空");
        assertTreatmentPlanUsed(id);
        return configTreatmentPlanService.removeById(id);
    }

    /**
     * 判断处置计划是否被使用
     * @param id 处置计划id
     * @return true:被使用 false:未被使用
     */
    public void assertTreatmentPlanUsed(String id){
        // 有没有被疾病关联
        boolean b = diseaseTreatmentPlanService.lambdaQuery()
                .eq(DiseaseTreatmentPlan::getTreatmentPlanId, id).count() > 0;
        Assert.isFalse(b,"处置计划已被疾病关联，不允许删除");

    }
    @Override
    public boolean updateTreatmentPlan(ConfigTreatmentPlan configDrug) {

        Assert.notEmpty(configDrug.getId(),"id 不允许为空");

        return configTreatmentPlanService.updateById(configDrug);
    }

    public void saveAssert(ConfigTreatmentPlan configDrug){
        Assert.isTrue(StrUtil.isEmpty(configDrug.getId()),"id 不为空");
        Assert.notNull(configDrug.getDisposalMethod(),"处置方式不允许为空");
        Assert.isTrue(configDrug.getDisposalMethod()==0 || configDrug.getDisposalMethod()==1,"处置方式不正确");

        Assert.isTrue(DisposalPlanEnum.containsCode(configDrug.getDisposalPlanId()),"处置计划不正确");

        Assert.notEmpty(configDrug.getFirstMeasures(),"一级措施不允许为空");

        //如果一级措施id为空，则判断一级措施是否已经存在，如果存在，则抛出异常
        if(StrUtil.isBlank(configDrug.getFirstMeasuresId())){
            Integer count = configTreatmentPlanService.lambdaQuery()
                    .eq(ConfigTreatmentPlan::getDisposalMethod, configDrug.getDisposalMethod())
                    .eq(ConfigTreatmentPlan::getDisposalPlanId, configDrug.getDisposalPlanId())
                    .eq(ConfigTreatmentPlan::getFirstMeasures, configDrug.getFirstMeasures()).count();
            Assert.isFalse(count>0,"一级措施已经存在");
        }
    }

}
